@@ -13,18 +13,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import json
+import logging
 import os
 
 import requests
 
-from barbicanclient.openstack.common import log as logging
 from barbicanclient.openstack.common.gettextutils import _
 from barbicanclient import orders
 from barbicanclient import secrets
+from barbicanclient import verifications
 
 
 LOG = logging.getLogger(__name__)
-logging.setup('barbicanclient')
 
 
 class HTTPError(Exception):
@@ -50,7 +50,8 @@ class HTTPAuthError(HTTPError):
 
 class Client(object):
 
-    def __init__(self, auth_plugin=None, endpoint=None, tenant_id=None):
+    def __init__(self, session=None, auth_plugin=None, endpoint=None,
+                 tenant_id=None, insecure=False):
         """
         Barbican client object used to interact with barbican service.
 
@@ -65,11 +66,19 @@ class Client(object):
         """
         LOG.debug(_("Creating Client object"))
 
-        self._session = requests.Session()
+        self._session = session or requests.Session()
+        self.verify = not insecure
         self.auth_plugin = auth_plugin
 
         if self.auth_plugin is not None:
-            self._barbican_url = self.auth_plugin.barbican_url
+            try:
+                self._barbican_url = self.auth_plugin.barbican_url
+            except:
+                if endpoint:
+                    self._barbican_url = endpoint
+                else:
+                    raise
+
             self._tenant_id = self.auth_plugin.tenant_id
             self._session.headers.update(
                 {'X-Auth-Token': self.auth_plugin.auth_token}
@@ -90,36 +99,39 @@ class Client(object):
         self.base_url = '{0}/{1}'.format(self._barbican_url, self._tenant_id)
         self.secrets = secrets.SecretManager(self)
         self.orders = orders.OrderManager(self)
+        self.verifications = verifications.VerificationManager(self)
 
     def get(self, href, params=None):
         headers = {'Accept': 'application/json'}
-        resp = self._session.get(href, params=params, headers=headers)
+        resp = self._session.get(href, params=params, headers=headers,
+                                 verify=self.verify)
         self._check_status_code(resp)
         return resp.json()
 
     def get_raw(self, href, headers):
-        resp = self._session.get(href, headers=headers)
+        resp = self._session.get(href, headers=headers, verify=self.verify)
         self._check_status_code(resp)
         return resp.content
 
     def delete(self, href):
-        resp = self._session.delete(href)
+        resp = self._session.delete(href, verify=self.verify)
         self._check_status_code(resp)
 
     def post(self, path, data):
         url = '{0}/{1}/'.format(self.base_url, path)
         headers = {'content-type': 'application/json'}
-        resp = self._session.post(url, data=json.dumps(data), headers=headers)
+        resp = self._session.post(url, data=json.dumps(data), headers=headers,
+                                  verify=self.verify)
         self._check_status_code(resp)
         return resp.json()
 
     def _check_status_code(self, resp):
-        status = resp.status_code
+        status = resp.status_code if resp else None
         LOG.debug('Response status {0}'.format(status))
         if status == 401:
             LOG.error('Auth error: {0}'.format(self._get_error_message(resp)))
             raise HTTPAuthError('{0}'.format(self._get_error_message(resp)))
-        if status >= 500:
+        if not status or status >= 500:
             LOG.error('5xx Server error: {0}'.format(
                 self._get_error_message(resp)
             ))
