@@ -19,6 +19,9 @@ import argparse
 
 from barbicanclient.common import auth
 from barbicanclient import client
+from barbicanclient.openstack.common import log as logging
+
+logging.setup('barbicanclient')
 
 
 class Keep:
@@ -33,6 +36,7 @@ class Keep:
         self._add_store_args()
         self._add_get_args()
         self._add_list_args()
+        self._add_verify_args()
         self._add_delete_args()
 
     def _get_main_parser(self):
@@ -41,9 +45,9 @@ class Keep:
         )
         parser.add_argument('command',
                             metavar='<entity>',
-                            choices=['order', 'secret'],
+                            choices=['order', 'secret', 'verification'],
                             help='Entity used for command, e.g.,'
-                                 ' order, secret.')
+                                 ' order, secret, verification.')
         auth_group = parser.add_mutually_exclusive_group()
         auth_group.add_argument('--no-auth', '-N', action='store_true',
                                 help='Do not use authentication.')
@@ -71,8 +75,37 @@ class Keep:
                             metavar='<barbican-url>',
                             default=client.env('BARBICAN_ENDPOINT'),
                             help='Defaults to env[BARBICAN_ENDPOINT].')
+        parser.add_argument('--insecure',
+                            default=False,
+                            action="store_true",
+                            help='Explicitly allow barbicanclient to perform '
+                                 '"insecure" TLS (https) requests. The '
+                                 'server\'s certificate will not be verified '
+                                 'against any certificate authorities. This '
+                                 'option should be used with caution.')
         return parser
 
+    def _add_verify_args(self):
+        verify_parser = self.subparsers.add_parser('verify',
+                                                   help='Create a new '
+                                                        'verification.')
+        verify_parser.add_argument('--type', '-t', default='image',
+                                   help='resource type to verify, '
+                                        'such as "image".')
+
+        verify_parser.add_argument('--ref', '-r',
+                                   help='reference URI to '
+                                        'resource to verify.')
+
+        verify_parser.add_argument('--action', '-a', default='vm_attach',
+                                   help='action to perform on '
+                                        'resource, such as "vm_attach".')
+
+        verify_parser.add_argument('--impersonation', '-i', default=True,
+                                   help='is impersonation allowed '
+                                        'for the resource.')
+        verify_parser.set_defaults(func=self.verify)
+
     def _add_create_args(self):
         create_parser = self.subparsers.add_parser('create',
                                                    help='Create a new order.')
@@ -93,7 +126,8 @@ class Keep:
                                    default='application/octet-stream',
                                    help='the type/format of the secret to be'
                                         ' generated (default: %(default)s).')
-        create_parser.add_argument('--expiration', '-x', help='the expiration '
+        create_parser.add_argument('--expiration', '-x',
+                                   help='the expiration '
                                    'time for the secret in ISO 8601 format.')
         create_parser.set_defaults(func=self.create)
 
@@ -105,7 +139,8 @@ class Keep:
         store_parser.add_argument('--name', '-n',
                                   help='a human-friendly name.')
         store_parser.add_argument('--payload', '-p', help='the unencrypted'
-                                  ' secret; if provided, you must also provide'
+                                  ' secret; if provided, '
+                                  'you must also provide'
                                   ' a payload_content_type')
         store_parser.add_argument('--payload-content-type', '-t',
                                   help='the type/format of the provided '
@@ -116,7 +151,8 @@ class Keep:
                                   help='required if --payload-content-type is'
                                   ' "application/octet-stream".')
         store_parser.add_argument('--algorithm', '-a', default='aes',
-                                  help='the algorithm (default: %(default)s).')
+                                  help='the algorithm (default: '
+                                       '%(default)s).')
         store_parser.add_argument('--bit-length', '-b', default=256,
                                   help='the bit length '
                                        '(default: %(default)s).',
@@ -131,19 +167,23 @@ class Keep:
     def _add_delete_args(self):
         delete_parser = self.subparsers.add_parser(
             'delete',
-            help='Delete a secret or an order by providing its href.'
+            help='Delete a secret, order or '
+                 'verification by providing its href.'
         )
         delete_parser.add_argument('URI', help='The URI reference for the'
-                                               ' secret or order')
+                                               ' secret, order '
+                                               'or verification')
         delete_parser.set_defaults(func=self.delete)
 
     def _add_get_args(self):
         get_parser = self.subparsers.add_parser(
             'get',
-            help='Retrieve a secret or an order by providing its URI.'
+            help='Retrieve a secret, order or '
+                 'verification by providing its URI.'
         )
-        get_parser.add_argument('URI', help='The URI reference for the secret'
-                                ' or order.')
+        get_parser.add_argument('URI', help='The URI reference '
+                                            'for the secret, '
+                                'order or verification.')
         get_parser.add_argument('--decrypt', '-d', help='if specified, keep'
                                 ' will retrieve the unencrypted secret data;'
                                 ' the data type can be specified with'
@@ -159,9 +199,11 @@ class Keep:
 
     def _add_list_args(self):
         list_parser = self.subparsers.add_parser('list',
-                                                 help='List secrets or orders')
-        list_parser.add_argument('--limit', '-l', default=10, help='specify t'
-                                 'he limit to the number of items to list per'
+                                                 help='List secrets, '
+                                                      'orders or '
+                                                      'verifications')
+        list_parser.add_argument('--limit', '-l', default=10, help='specify '
+                                 'the limit to the number of items to list per'
                                  ' page (default: %(default)s; maximum: 100)',
                                  type=int)
         list_parser.add_argument('--offset', '-o', default=0, help='specify t'
@@ -200,8 +242,14 @@ class Keep:
     def delete(self, args):
         if args.command == 'secret':
             self.client.secrets.delete(args.URI)
-        else:
+        elif args.command == 'verification':
+            self.client.verifications.delete(args.URI)
+        elif args.command == 'order':
             self.client.orders.delete(args.URI)
+        else:
+            self.parser.exit(status=1, message='ERROR: delete is only '
+                                               'supported for secrets, '
+                                               'orders or verifications\n')
 
     def get(self, args):
         if args.command == 'secret':
@@ -210,35 +258,62 @@ class Keep:
                                                   args.payload_content_type)
             else:
                 print self.client.secrets.get(args.URI)
-        else:
+        elif args.command == 'verification':
+            print self.client.verifications.get(args.URI)
+        elif args.command == 'order':
             print self.client.orders.get(args.URI)
+        else:
+            self.parser.exit(status=1, message='ERROR: get is only '
+                                               'supported for secrets, '
+                                               'orders or verifications\n')
 
     def list(self, args):
         if args.command == 'secret':
             ls = self.client.secrets.list(args.limit, args.offset)
-        else:
+        elif args.command == 'verification':
+            ls = self.client.verifications.list(args.limit, args.offset)
+        elif args.command == 'order':
             ls = self.client.orders.list(args.limit, args.offset)
+        else:
+            self.parser.exit(status=1, message='ERROR: get list is only '
+                                               'supported for secrets, '
+                                               'orders or verifications\n')
         for obj in ls:
             print obj
         print '{0}s displayed: {1} - offset: {2}'.format(args.command, len(ls),
                                                          args.offset)
 
+    def verify(self, args):
+        if args.command == 'verification':
+            verify = self.client.verifications\
+                .create(resource_type=args.type,
+                        resource_ref=args.ref,
+                        resource_action=args.action,
+                        impersonation_allowed=args.impersonation)
+            print verify
+        else:
+            self.parser.exit(status=1, message='ERROR: verify is only '
+                                               'supported for verifications\n')
+
     def execute(self, **kwargs):
         args = self.parser.parse_args(kwargs.get('argv'))
         if args.no_auth:
             self.client = client.Client(endpoint=args.endpoint,
-                                        tenant_id=args.os_tenant_id)
+                                        tenant_id=args.os_tenant_id,
+                                        insecure=args.insecure)
         elif all([args.os_auth_url, args.os_username, args.os_password,
                   args.os_tenant_name]):
             self._keystone = auth.KeystoneAuthV2(
                 auth_url=args.os_auth_url,
                 username=args.os_username,
                 password=args.os_password,
-                tenant_name=args.os_tenant_name
+                tenant_name=args.os_tenant_name,
+                insecure=args.insecure
             )
             self.client = client.Client(auth_plugin=self._keystone,
                                         endpoint=args.endpoint,
-                                        tenant_id=args.tenant_id)
+                                        tenant_id=args.os_tenant_id,
+                                        insecure=args.insecure)
         else:
             self.parser.exit(
                 status=1,
