@@ -12,8 +12,9 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import logging
+
 from barbicanclient import base
-from barbicanclient.openstack.common import log as logging
 from barbicanclient.openstack.common.timeutils import parse_isotime
 
 
@@ -139,6 +140,12 @@ class SecretManager(base.BaseEntityManager):
             raise ValueError('secret_ref is required.')
         if not content_type:
             secret = self.get(secret_ref)
+            if secret.content_types is None:
+                raise ValueError('Secret has no encrypted data to decrypt.')
+            if 'default' not in secret.content_types:
+                raise ValueError("Must specify decrypt content-type as "
+                                 "secret does not specify a 'default' "
+                                 "content-type.")
             content_type = secret.content_types['default']
         headers = {'Accept': content_type}
         return self.api.get_raw(secret_ref, headers)
